/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dashboard.
 *
 * The Initial Developer of the Original Code is
 *   Dave Raggett.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

// as this is called from the overlay, we get one per window
// but we may want to instead stick to one per browser, hmmm

// see also https://developer.mozilla.org/en/XPCOM_Interface_Reference/NsITraceableChannel
// and http://www.softwareishard.com/blog/firebug/nsitraceablechannel-intercept-http-traffic/

dashboard_overlay.register_observer({
  active: false,
  timer: null,
  database: null,

  start_up: function (context)
  {
    this.database = dashboard_overlay.database;
    this.context = context; // single instance of dashboard context for all windows

    // ensure that only one set of observers exists for all windows
    // note: context is a dashboard session object on a singleton window
    if (context.window_count == 0)
    {
       let observerService =
          Components.classes["@mozilla.org/observer-service;1"]
                    .getService(Components.interfaces.nsIObserverService);
      observerService.addObserver(this, "http-on-modify-request", false);
      observerService.addObserver(this, "http-on-examine-response", false);
      observerService.addObserver(this, "cookie-changed", false);

      // register content policy

      let registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
      let policy = this.policy;

      try
      {
       registrar.registerFactory(policy.classID,
                        policy.classDescription, 
                        policy.contractID, policy);
      }
      catch (e)
      {
        Cu.reportError(e);
      }

      let category_manager = Cc["@mozilla.org/categorymanager;1"]
                              .getService(Ci.nsICategoryManager);  
      category_manager.addCategoryEntry("content-policy",
          policy.classDescription, policy.contractID, false, true);
      }
  },

  // if this window hosts the observer, remove it
  // if this window is not the only window pass ownership
  shut_down: function (context)
  {
    if (context.window_count == 0)
    {
      let observerService =
         Components.classes["@mozilla.org/observer-service;1"]
                   .getService(Components.interfaces.nsIObserverService);

      observerService.removeObserver(this, "http-on-examine-response");
      observerService.removeObserver(this, "http-on-modify-request");
      observerService.addObserver(this, "cookie-changed", false);

      // is it sensible to delete the category entry here?
      let category_manager = Cc["@mozilla.org/categorymanager;1"]
                              .getService(Ci.nsICategoryManager);  
      category_manager.deleteCategoryEntry("content-policy",
                                 this.policy.classDescription, false);

      // flush any log data to the database
      this.dump();
    }
  },

  // XPCOM interface for content-policy notifications that
  // allows us to scrutinize requests to load pages, scripts,
  // etc. before the http request is generated for them
  policy:
  {
    classDescription: "Privacy Dashboard content policy",
    classID: Components.ID("9762b15c-f233-11df-9eb4-002170f348ec"),
    contractID: "@dave.raggett/policy;1",

    // nsISupports interface implementation

    QueryInterface: function(aIID)  
    {
      if (aIID.equals(Ci.nsIContentPolicy) ||  
          aIID.equals(Ci.nsISupportsWeakReference) ||  
          aIID.equals(Ci.nsISupports))  
        return this;
      throw Cr.NS_NOINTERFACE;  
    },

    // nsIFactory interface implementation
    createInstance: function(outer, iid)
    {
      if (outer)
        throw Cr.NS_ERROR_NO_AGGREGATION;

      return this.QueryInterface(iid);
    },

    shouldLoad: function(contentType, contentLocation,
                 requestOrigin, node, mimeTypeGuess, extra)
    {
      
      if (contentType == Components.interfaces.nsIContentPolicy.TYPE_DOCUMENT &&
             /^http.+/.test(contentLocation.spec) &&
             node.nodeName == "xul:browser")
      {
        node.dashboard_status = 
          dashboard_overlay.observer.init_status(contentLocation);
      }

      return Components.interfaces.nsIContentPolicy.ACCEPT;
    },

    shouldProcess: function(contentType, contentLocation,
                   requestOrigin, insecNode, mimeType, extra)
    {
      return Components.interfaces.nsIContentPolicy.ACCEPT;
    },
  },

  // called by the observer service
  observe: function(subject, topic, data)
  {
    let observer = dashboard_overlay.observer;

    if (topic == "http-on-modify-request")
    {
      let httpChannel = subject.QueryInterface(Components.interfaces.nsIHttpChannel);
      let index = observer.get_tab_index_from_channel(httpChannel);
      if (index < 0) console.log("*** couldn't get tab for " + httpChannel.URI.spec);
      let browser = index >= 0 ? gBrowser.getBrowserAtIndex(index) : null;

      // browser is null for requests not involved in page load
      // e.g. browser p3p request, favicon, and safe browsing
      // we avoid logging these for now ....

      if (!browser)
      {
        if (httpChannel.referrer)
          console.log("*** couldn't get browser, but referrer is " + httpChannel.referrer.spec);
        else
          console.log("*** couldn't get browser on http-on-modify-request for " + httpChannel.URI.spec);
      }

      if (!browser)
        return;

      // due to threading and weird delays before notications
      // are delivered, we can sometimes get the http request 
      // notification *before* policy shouldLoad, and when
      // this happens we need to initialize the status here

      let status = browser.dashboard_status;

      if (!status) // we've been called before shouldLoad
      {
        status = browser.dashboard_status = 
          dashboard_overlay.observer.init_status(httpChannel.URI);
        console.log("observe: creating status for " + httpChannel.URI.spec);
      }

      // deal with page redirects
      if (status.new_url && httpChannel.URI.spec == status.new_url)
      {
        //alert("responding to redirect to " + httpChannel.URI.host);
        status.old_uri = status.uri;
        status.old_host = status.page_host;
        status.uri = httpChannel.URI;
        status.page_host = httpChannel.URI.host;
        status.new_url = null;
      }

      let prefs = status.prefs;
      let request_domain = dashboard_overlay.base_domain(httpChannel.URI);
      let context = dashboard_overlay.context;
      let target = httpChannel.URI;
      let referrer = httpChannel.referrer;

      let offsite = 0;
      let block_3rd_party_cookie = false;
      let block_3rd_parties = false;

      // the "do not track" header, as defined in
      // http://www.w3.org/Submission/web-tracking-protection/
      if (prefs & (1 << 15))
      {
        httpChannel.setRequestHeader("DNT", "1", false);
      }

      // unwanted cookies are purged after page has loaded

      // 3 from bit position in assess:save_prefs   
      block_3rd_parties = prefs & (1<<3);

      // look for 3rd party requests
      if (status.page_host && status.page_host != target.host)
      {
        offsite = 1; // internal third party request

        if (status.page_domain != request_domain)
        {
          offsite = 2;  // external third party request

          if (prefs & (1 << 4)) // 4 from bit position in assess:save_prefs  
            block_3rd_party_cookie = true;

          if (block_3rd_parties)
          {
            offsite = 2;
            //alert("blocked: " + httpChannel.URI.spec);

            let rp = dashboard_overlay.host_prefs(target.host);
            let never = rp & (1 <<2); // never block content from this site

            try
            {
              if (!never)
                httpChannel.cancel(-1); //(NS_BINDING_ABORTED);  // -1
            } 
            catch (e)
            {
              alert("couldn't cancel http request: " + e);
            }
          }
        }
      }

      // build set of direct and indirect 3rd parties for this page

      let referrer_host = 
        (referrer && /^http.+/.test(referrer.spec) ? referrer.host : null);
      
      if (referrer_host && offsite)
      {
        if (referrer_host == status.page_host || status.parties[referrer_host])
        {
          status.parties[target.host] = offsite;
          status.pty.push(target.host);
        }

        //alert("now " + status.pty.length + " 3rd parties at " + Date.now());
      }

      // docURI is the previous page for the http request for a new page
      // and the prefs aren't available until after the onlocationchange event

      let record = new Object();
      record.uri = target.spec;
      record.host = target.host;
      //record.path = target.path;
      record.referrer = (httpChannel.referrer ? httpChannel.referrer.spec : null);
      record.ref_host = (httpChannel.referrer ? httpChannel.referrer.host : null);
      //record.basedomain = request_domain;
      record.page_host = status.page_host;
      //record.method = httpChannel.requestMethod;
      record.offsite = offsite;

      try
      {
        let d = httpChannel.getRequestHeader("If-Modified-Since")
        record.cache_date = this.iso8061(new Date(d));
       }
      catch (e)
      { }

      try
      {
        record.content_type = httpChannel.getRequestHeader("Content-Type");
      }
      catch (e) 
      { }

      record.path = target.path;
      record.query_params = [];
      record.post_params = [];

      // with name/values pairs in URI query string
      let query = this.query_string(record.path);

      if (query)
      {
        let trim = dashboard_overlay.trim;
        let params = query.split("&");

        for (let j = 0; j < params.length; ++j)
        {
          let param = params[j].split("=");
          let name = trim(param[0]);
          let value = param[1];
          let form = this.is_form_field_name(record.browser, name);
          record.query_params.push({"name": name, "value": value, "form": form});
        }
      }

      // how to determine if the request contains a HTML Form data body?
      // POST and no Content-Type header and same URI as submit event?

      // code to read the POST body
      if (httpChannel.requestMethod == "POST")
      {
        httpChannel.QueryInterface(Components.interfaces.nsIUploadChannel);
        httpChannel.uploadStream.QueryInterface(Components.interfaces.nsISeekableStream);
        var stream = Components.classes["@mozilla.org/scriptableinputstream;1"] 
                         .createInstance(Components.interfaces.nsIScriptableInputStream); 
        stream.init(httpChannel.uploadStream);
        httpChannel.uploadStream.seek(Components.interfaces.nsISeekableStream.NS_SEEK_SET, 0);

        // read from stream: stream.read(...)
        let len = stream.available();

        // first read the content type which should be
        // "Content-Type: application/x-www-form-urlencoded"
        // following assumes single space and \r\n at end of line
        let buf = stream.read(49);

        // check if it is form data
        if (/application\/x\-www\-form\-urlencoded/.test(buf))
        {    
          buf += stream.read(len-49);

          // risky if requests don't use \r\n properly
          let start = buf.indexOf("\r\n\r\n");

          if (start > 0)
            record.post_data = buf.substr(start+4);

          // for simple form this gives something like
          // your_name=Fred+Flintstone&userid=marvin&form_name=turingtest
        }

        // and rewind the stream
        httpChannel.QueryInterface(Components.interfaces.nsIUploadChannel); 
        httpChannel.uploadStream.QueryInterface(Components.interfaces.nsISeekableStream);
        httpChannel.uploadStream.seek(Components.interfaces.nsISeekableStream.NS_SEEK_SET, 0);

        if (record.post_data)
        {
          let trim = dashboard_overlay.trim;
          let params = record.post_data.split("&");

          for (let j = 0; j < params.length; ++j)
          {
            let param = params[j].split("=");
            let name = trim(param[0]);
            let value = param[1];
            let form = this.is_form_field_name(record.browser, name);
            record.post_params.push({"name": name, "value": value, "form": form});
          }
        }
      }

      this.requests.push(record);

      let self = this;  // used to ensure 'this' is correct in dump()
      this.timer = setTimeout(function() {self.dump();}, 100);
    }
    else if (topic == "http-on-examine-response")
    {
      let httpChannel = subject.QueryInterface(Components.interfaces.nsIHttpChannel);

      let type = "";
      try {
        type = httpChannel.getResponseHeader("Content-Type");
      } catch (e) {
        type = "undefined";
      } 

      console.log("loading (" + httpChannel.responseStatus +  ") " + type + " from " + httpChannel.URI.spec);

      if (httpChannel && httpChannel.responseStatus == 200)
      {
        let index = observer.get_tab_index_from_channel(httpChannel);

        let browser = index >= 0 ? gBrowser.getBrowserAtIndex(index) : null;

        if (browser)
        {
          let status = browser.dashboard_status;
          let target = httpChannel.URI;

          let rp = dashboard_overlay.host_prefs(target.host);
          let never = rp ? rp & (1 <<2) : false ; // never block content from this site

          let block_lasting_cookies = status.prefs & (1<<5);
          let block_3rd_parties = status.prefs & (1<<3);
          let block_flash = status.prefs & (1<<12);
          let block_java = status.prefs & (1<<13);

          // always block cookies set on P3P policy!
          if (target.path == "/w3c/p3p.xml")
            observer.block_p3p_cookies(httpChannel);

          if (block_3rd_parties & !never)
            observer.block_3rd_party_content(httpChannel);
          else
          {
            if (block_flash & !never)
              observer.block_flash_content(httpChannel);

            if (block_java && !never)
              observer.block_java_content(httpChannel);
          }
        }
        else if (httpChannel.referrer)
        {
          // could not determine tab, so fall back to referrer
          let host = httpChannel.referrer.host;

          if (host)
          {
            let prefs = dashboard_overlay.host_prefs(host);
            let target = httpChannel.URI;

            let rp = dashboard_overlay.host_prefs(target.host);
            let never = rp ? rp & (1 <<2) : false ; // never block content from this site

            let block_3rd_parties = prefs & (1<<3);
            let block_flash = prefs & (1<<12);
            let block_java = prefs & (1<<13);

            if (block_3rd_parties & !never)
              observer.block_3rd_party_content(httpChannel);
            else
            {
              if (block_flash && !never)
                observer.block_flash_content(httpChannel);

              if (block_java && !never)
                observer.block_java_content(httpChannel);
            }
          }
        }
      }
    }
    else if (topic == "cookie-changed")  // only used for logging purposes
    {
      // subject is nsICookie2 or nsIArray of nsICookie2 objects
      // data  is "deleted", "added", "changed", "batch-deleted", "cleared or "reload"
      let cookies = null;

      try {
        cookies = subject.QueryInterface(Components.interfaces.nsIArray);
      } catch (e) {
        cookies = null;
      }

      if (!cookies)
      {
        let cookie = subject ? subject.QueryInterface(Components.interfaces.nsICookie2) : null;

        if (cookie)
          cookies = [ cookie ];
      }

      let i = 0;
      while (i < cookies.length)
      {
        let cookie = cookies[i++];
        console.log("cookie " + cookie.name + " for " + cookie.host + " " + data);
      }
    }
  },

  parties: {},             // set of third parties for page
  requests: new Array(),   // list of http request records

  // determine which tab is associated with given http request
  // this returns -1 for requests initiated by the browser
  // e.g. p3p, favicon, safe browsing list, and sometimes for
  // other requests for resources in web pages, :-(
  get_tab_index_from_channel: function (aChannel)
  {
    try
    {
      if (aChannel.notificationCallbacks)
      {
        // this often generates an exception - no such interface
        // if that happens, try again with load group
        let ir = aChannel.notificationCallbacks.
             QueryInterface(Components.interfaces.nsIInterfaceRequestor);

        let doc = ir.
           getInterface(Components.interfaces.nsIDOMWindow).document;

        return gBrowser.getBrowserIndexForDocument(doc);
      }

      console.log("get_tab_index_from_channel: notificationCallbacks is null");
      return -2;
    }
    catch (e)
    {
      console.log("get_tab_index_from_channel: " + e);
      if (aChannel.loadGroup)
      {
        if (aChannel.loadGroup.notificationCallbacks)
        {
          try 
          {
            let ir = aChannel.loadGroup.notificationCallbacks.
               QueryInterface(Components.interfaces.nsIInterfaceRequestor);

            let doc = ir.
               getInterface(Components.interfaces.nsIDOMWindow).document;

            // the index is occasionally -1 for top level request
            // for page, but thereafter gives correct result
            return gBrowser.getBrowserIndexForDocument(doc);
          }
          catch (e)
          {
            console.log("get_tab_index_from_channel: " + e);
            return -4;
          }
        }
      }

      console.log("get_tab_index_from_channel: notificationCallbacks is null or loadGroup is null");
      return -3;
    }
  },

  block_p3p_cookies: function (httpChannel)
  {
    console.log("observer.block_p3p_cookies on " + httpChannel.URI.host);

    let visitor = {
      "channel": httpChannel,
      "host": httpChannel.URI.host,

      "visitHeader": function (header, value)
      {
        if (header == "Set-Cookie")
          this.channel.setResponseHeader(header, null, false);
      }
    };

    httpChannel.visitResponseHeaders(visitor);
  },

  // assumes that existence of referrer has already been checked
  block_3rd_party_content: function (httpChannel)
  {
    let page_host = httpChannel.referrer.host;

    if (!page_host)
      return false;

    let page_domain = dashboard_overlay.base_domain(page_host);
    let target_domain = dashboard_overlay.base_domain(httpChannel.URI.host);

    if (page_domain != target_domain)
    {
      try
      {
        httpChannel.cancel(-1);  //(NS_BINDING_ABORTED);  // -1
        console.log("blocking flash content");
      } 
      catch (e)
      {
        console.log("couldn't cancel http request: " + e);
      }

      return true;
    }

    return false;
  },

  block_flash_content: function (httpChannel)
  {
    let type = null;

    try {
      type = httpChannel.getResponseHeader("Content-Type");
    } catch (e) { type = ""; }

    if (type && (type.indexOf("flash") != -1 ||
                 type.indexOf("shockwave") != -1 ||
                 type.indexOf("futuresplash") != -1))
    {
      try
      {
        httpChannel.cancel(-1);  //(NS_BINDING_ABORTED);  // -1
        console.log("blocking flash content");
      } 
      catch (e)
      {
        console.log("couldn't cancel http request: " + e);
      }

      return true;
    }

    return false;
  },

  block_java_content: function (httpChannel)
  {
    let type = httpChannel.getResponseHeader("Content-Type");

    if (type && type.indexOf("java-") != -1)
    {
      try
      {
        httpChannel.cancel(-1); // (NS_BINDING_ABORTED);  // -1
        console.log("blocking java content");
      } 
      catch (e)
      {
        console.log("couldn't cancel http request: " + e);
      }

      return true;
    }

    return false;
  },

  init_status: function (uri)
  {
    let scheme = uri.scheme;
    let status = {};
    status.loaded = false;
    status.uri = uri;
    status.page_host = /^http.+/.test(uri.spec) ? uri.host : null;
    status.page_domain = dashboard_overlay.base_domain(uri);
    status.prefs = dashboard_overlay.host_prefs(status.page_host);
    status.parties = {};
    status.pty = [];
    status.session_cookies = 0;
    status.lasting_cookies = 0;
    status.flash_cookies = 0;
    status.int_3rd_parties = 0;
    status.ext_3rd_parties = 0;
    status.int_3rd_party_session_cookies = 0;
    status.int_3rd_party_lasting_cookies = 0;
    status.int_3rd_party_flash_cookies = 0;
    status.ext_3rd_party_session_cookies = 0;
    status.ext_3rd_party_lasting_cookies = 0;
    status.ext_3rd_party_flash_cookies = 0;
    status.dom_storage = 0;
    status.html5_pings = 0;
    status.invisible_images = 0;
    status.suspicious_urls = 0;
    status.geo_permission = 0;
    status.p3p = 0;
    status.visited = 0;

    var prefs = status.prefs;

    // not sure of the interfaces used by apply_prefs
    // are safe to call from shouldLoad, so play safe
    // and apply browser preferences in clean context

    setTimeout( function ()
    {
      dashboard_overlay.apply_prefs(prefs);
    }, 0);
    return status;
  },

  query_string: function (path) 
  {
    if (path)
    {
      let index = path.indexOf("?");

      if (index >= 0)
        return path.substr(index+1);
    }

    return null;
  },

  // return 1 if name is that of form field for this tab
  is_form_field_name: function (browser, name)
  {
    if (!browser)
      return 0;

    let doc = browser.contentDocument;

    if (!doc)
      return 0;

    let forms = doc.forms;

    if (!forms)
      return 0;

    for (let i = 0; i < forms.length; ++i)
    {
      let fields = forms[i].elements;

      //if (fields[name])  // boo hoo doesn't work
      //  return 1;

      for (let j = 0; j < fields.length; ++j)
      {
        let fieldName = fields[j].name;

        if (fieldName && fieldName == name)
          return 1;
      }
    }

    return 0;
  },

  // log records to database and clear them
  // doc.baseURIObject is the nsURI object for the page
  dump: function ()
  {
    if (this.timer)
    {
      clearTimeout(this.timer);
      this.timer = null;
    }

    // check if there is something to log
    if (this.requests.length > 0)
    {
      this.database.log_http(this.requests);

      if (this.requests.length > 0)
        this.requests = new Array(); // reset
    }
  },

  QueryInterface : function(aIID)
  {  
    if (aIID.equals(Components.interfaces.nsISupports) ||  
        aIID.equals(Components.interfaces.nsIObserver))  
      return this;  
    throw Components.results.NS_NOINTERFACE;  
  }  
});

