/*
 *      main.c -- Analysis program for dashbot data in SQLite
 *
 *      Copyright (c) 2011 Dave Raggett
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <assert.h>
#include <string.h>   // for strlen
#include <strings.h>
#include <sqlite3.h>

#include "analyst.h"
#include "top_sites.h"

#define DATABASE "dashboard.sqlite"

// intro to SQLite API:  http://www.sqlite.org/cintro.html

char *prog_name;
int debug;

void main_init(int argc, char **argv)
{
  prog_name = argv[0];
  char *p = strrchr(prog_name, '/');
  if (p)
    prog_name = ++p;
}
// 1st arg is context passed in 4th arg of sqlite3_exec
// 2nd arg is number of columns
// 3rd arg is an array of pointers to column values
// 4th arg is an array of pointers to column names
// note: column value may be null so check for null pointer
static int callback(void *NotUsed, int argc, char **argv, char **azColName)
{
  int i;

  for(i=0; i<argc; i++)
  {
    printf("%s = %s\n", azColName[i], argv[i] ? argv[i] : "NULL");
  }

  printf("\n");
  return 0;
}

static void test(sqlite3 *db)
{
  char *zErrMsg = 0;
  char *sql = "SELECT DISTINCT host FROM site_info WHERE lasting_cookies <> 0";

  int rc = sqlite3_exec(db, sql, callback, null, &zErrMsg);

  if( rc != SQLITE_OK )
  {
    fprintf(stderr, "SQL error: %s\n", zErrMsg);
    sqlite3_free(zErrMsg);
  }
}

static void test_top_sites(int index)
{
  SiteDesc desc = top_sites[index];

  printf("%d: rank %ld - %s - %s - %ld visitors\n",
         index, desc.rank, desc.domain, desc.category, desc.visitors);
}

int main(int argc, char **argv)
{
  sqlite3 *db;
  char *database = DATABASE;

  main_init(argc, argv);

  if( argc!=1 )
  {
    fprintf(stderr, "Usage: %s\n a program to analyse privacy dashboard data in SQLite\n", prog_name);
    exit(1);
  }

  //test_top_sites(38);

  if(sqlite3_open(database, &db))
  {
    fprintf(stderr, "Can't open database %s: %s\n", database, sqlite3_errmsg(db));
    sqlite3_close(db);
    exit(1);
  }

  //test(db);
  graph(db);
  gml(db);
  study_clusters(db);

  sqlite3_close(db);
  return 0;
}
