window.onload = start;


function start()
{
  dashboard_p3p.get_policy("http://localhost/w3c/bp.xml", "/",
    dashboard_p3p.find_applicable_policy);
}

var dashboard_p3p = {
  get_policy: function(policy_uri, path, handler) {
    var req = new XMLHttpRequest();

    req.open("GET", policy_uri, true);
    req.onreadystatechange = function (evt) {

      if (req.readyState == 4)
      {
        if (req.status == 200)
          handler(path, req);
      }
    };
    req.send(null);
  },

// from XUL overlay, we can access req.channel.URI.resolve(rel_url)
// but not from regular web page script, so we use hack for now

  find_applicable_policy: function (path, req) {
    var root = req.responseXML.documentElement;
    var policy_refs = root.getElementsByTagName("POLICY-REF");

    if (policy_refs && policy_refs.length > 0)
    {
      var uri = policy_refs[0].getAttribute("about");
      var hash = uri.indexOf("#");

      if (hash >= 0)
        uri = uri.substr(0, hash);

      uri = "http://localhost/w3c/" + uri;      

      dashboard_p3p.get_policy(uri, path,
         dashboard_p3p.find_applicable_policy);
    }
    else
    {
      // otherwise render policy
      dashboard_p3p.render_policy(path, root);
    }
  },

  render_policy: function (path, root) {
    var p3p = dashboard_p3p;

    var policies = root.getElementsByTagName("POLICY");
    var policy = policies ? policies[0] : null;

    if (!policy)
      return;

    // use server's own rendering of policy
    if (false)
    {
       var discuri = policy.getAttribute("discuri");

       if (discuri)
       {
         location.href = discuri;
         return;
       }
    }

    // deal with contact info
    var contact = p3p.get_contact_group(policy);
    p3p.dump_contact(contact, "contact");

    // deal with access info
    var access = policy.getElementsByTagName("ACCESS");

    if (access && access.length > 0)
      access = access[0];

    // deal with dispute info
    // hmm, let's ignore this for now

    // deal with statements
    var statements = policy.getElementsByTagName("STATEMENT");
    for (var i = 0; i < statements.length; ++i)
      p3p.dump_statement(statements[i]);
  },

  get_contact_group: function (policy) {
    var ents = policy.getElementsByTagName("ENTITY");
  
    if (ents && ents.length > 0)
    {
      var dg = ents[0].getElementsByTagName("DATA-GROUP");
 
      if (dg && dg.length > 0)
        return dg[0];
    }

    return null;
  },

  dump_contact: function (contact, id) {
    var obj = {};

    for (var data = contact.firstChild; data; data = data.nextSibling)
    {
      if (data.nodeType == 1)
      {
        var ref = data.getAttribute("ref");
        var value = data.firstChild.nodeValue;
        this.analyse_info(obj, ref, value);
      }
    }

    var div = document.getElementById(id);
    var s = "";

    s += "<h2>"+obj.business.name+" Privacy Policy</h2>\n<p>\n";

    s += obj.business.contact_info.postal.street + "<br />\n";
    s += obj.business.contact_info.postal.city + ", ";

    if (obj.business.contact_info.postal.stateprov)
      s += obj.business.contact_info.postal.stateprov + " ";
  
    if (obj.business.contact_info.postal.postalcode)
      s += obj.business.contact_info.postal.postalcode + "<br />\n";

    s += obj.business.contact_info.postal.country + "<br />\n";

    if (obj.business.contact_info.online.email)
    {
      s += "<br \>email: \n";
      s += obj.business.contact_info.online.email;
    }

    if (obj.business.contact_info.telecom.telephone)
    {
      s += "<br \>phone: \n";
      if (obj.business.contact_info.telecom.telephone.intcode)
      {
        s += "+"  + obj.business.contact_info.telecom.telephone.intcode;
        s += "."  + obj.business.contact_info.telecom.telephone.loccode;
        s += "."  + obj.business.contact_info.telecom.telephone.number;
      }
      else
      {
         if (obj.business.contact_info.telecom.telephone.substr(0,2) == "00")
           s += "+" + obj.business.contact_info.telecom.telephone.substr(2);
         else
           s += obj.business.contact_info.telecom.telephone;
      }
    }

    s += "\n</p>\n";

    div.innerHTML += s;
  },

  // map ref string into object tree
  analyse_info: function (obj, ref, value) {
    ref = ref.substr(1).split(".");

    var part = obj;
    var name;

    for (var i = 0; i < ref.length - 1; ++i)
    {
      name = ref[i];
      name = name.replace(/-/g, "_");

      if (!part[name])
        part[name] = {};

      part = part[name];
    }

    name = ref[ref.length - 1];
    part[name] = value;
  },

  dump_statement: function (statement) {
    var div = document.createElement("div");

    var p = document.createElement("p");
    p.innerHTML = this.get_consequence(statement);
    div.appendChild(p);

    p = document.createElement("p");
    p.innerHTML = "The follow kinds of data are collected";
    div.appendChild(p); 

    var ul = document.createElement("ul");
    div.appendChild(ul); 

    var el = statement.getElementsByTagName("DATA-GROUP");

    if (el && el.length > 0)
    {
      var data = el[0].getElementsByTagName("DATA");
      var li;
      var desc;

      for (var i = 0; i < data.length; ++i)
      {
        var ref = data[i].getAttribute("ref").substr(1);
        var desc = this.get_description(ref);
        if (!desc) desc = ref;
        li = document.createElement("li");
        li.innerHTML = desc;
        ul.appendChild(li);
      }
    }

    p = document.createElement("p");
    p.innerHTML = "This data is used for";
    div.appendChild(p); 

    var ul = document.createElement("ul");
    ul.className = "long";
    div.appendChild(ul); 

    el = statement.getElementsByTagName("PURPOSE");

    if (el && el.length > 0)
    {
      for (el = el[0].firstChild; el; el = el.nextSibling)
      {
        if (el.nodeType != 1)
          continue;

        var purpose = this.purposes[el.nodeName];
        if (!purpose) purpose = el.nodeName;

        var li = document.createElement("li");
        li.innerHTML = purpose+this.opt_in_out(el);
        ul.appendChild(li);
      }
    }

    p = document.createElement("p");
    p.innerHTML = "This data is used by";
    div.appendChild(p); 

    var ul = document.createElement("ul");
    ul.className = "long";
    div.appendChild(ul); 

    el = statement.getElementsByTagName("RECIPIENT");

    if (el && el.length > 0)
    {
      for (el = el[0].firstChild; el; el = el.nextSibling)
      {
        if (el.nodeType != 1)
          continue;

        var recipient = this.recipients[el.nodeName];
        if (!recipient) purpose = el.nodeName;

        var li = document.createElement("li");
        li.innerHTML = recipient+this.opt_in_out(el)+
           this.recipient_description(el);
        ul.appendChild(li);
      }
    }

    p = document.createElement("p");
    p.innerHTML = "Data retention:";
    div.appendChild(p);

    var ul = document.createElement("ul");
    ul.className = "long";
    div.appendChild(ul); 

    el = statement.getElementsByTagName("RETENTION");

    if (el && el.length > 0)
    {
      for (el = el[0].firstChild; el; el = el.nextSibling)
      {
        if (el.nodeType != 1)
          continue;

        var retention = this.retention[el.nodeName];
        if (!retention) purpose = el.nodeName;

        var li = document.createElement("li");
        li.innerHTML = retention;
        ul.appendChild(li);
      }
    }

    document.body.appendChild(div);
  },

  opt_in_out: function (el) {
    var req = el.getAttribute("required");

    if (req == "opt-in")
      return " (opt in)";

    if (req == "opt-out")
      return " (opt out)";

    return "";
  },

  recipient_description: function (el) {
    el = el.firstElementChild;

    if (el)
      return ", but note that " + el.firstChild.nodeValue;

    return "";
  },

  get_consequence: function (statement) {
    var el = statement.getElementsByTagName("CONSEQUENCE");

    if (el && el.length > 0)
      return el[0].firstChild.nodeValue;

    return null;
  },

  purposes: {
    "current" :
      "Completion and Support of Activity For Which Data Was " +
      "Provided<span>: Information may be used by the service provider " +
      "to complete the activity for which it was provided, whether " +
      "a one-time activity such as returning the results from a Web " +
      "search, forwarding an email message, or placing an order; or " +
      "a recurring activity such as providing a subscription service, " +
      "or allowing access to an online address book or electronic " +
      "wallet.</span>",
    "admin" :
      "Web Site and System Administration<span>: Information may be used " +
      "for the technical support of the Web site and its computer " +
      "system. This would include processing computer account " +
      "information, information used in the course of securing and " +
      "maintaining the site, and verification of Web site activity " +
      "by the site or its agents.</span>",
    "develop" :
      "Research and Development<span>: Information may be used to " +
      "enhance, evaluate, or otherwise review the site, service, " +
      "product, or market. This does not include personal information " +
      "used to tailor or modify the content to the specific individual " +
      "nor information used to evaluate, target, profile or contact " +
      "the individual.</span>",
    "tailoring" :
      "One-time Tailoring<span>: Information may be used to tailor " +
      "or modify content or design of the site where the information " +
      "is used only for a single visit to the site and not used for " +
      "any kind of future customization. For example, an online store " +
      "might suggest other items a visitor may wish to purchase based " +
      "on the items he has already placed in his shopping basket.</span>",
    "pseudo-analysis" :
      "Pseudonymous Analysis<span>: Information may be used to create " +
      "or build a record of a particular individual or computer that " +
      "is tied to a pseudonymous identifier, without tying identified " +
      "data (such as name, address, phone number, or email address) " +
      "to the record. This profile will be used to determine the habits, " +
      "interests, or other characteristics of individuals for purpose " +
      "of research, analysis and reporting, but it will not be used to " +
      "attempt to identify specific individuals. For example, a marketer " +
      "may wish to understand the interests of visitors to different " +
      "portions of a Web site.</span>",
    "pseudo-decision" :
      "Pseudonymous Decision<span>: Information may be used to create " +
      "or build a record of a particular individual or computer that " +
      "is tied to a pseudonymous identifier, without tying identified " +
      "data (such as name, address, phone number, or email address) to " +
      "the record. This profile will be used to determine the habits, " +
      "interests, or other characteristics of individuals to make a " +
      "decision that directly affects that individual, but it will not " +
      "be used to attempt to identify specific individuals. For example, " +
      "a marketer may tailor or modify content displayed to the browser " +
      "based on pages viewed during previous visits.</span>",
    "individual-analysis" :
      "Individual Analysis<span>: Information may be used to determine " +
      "the habits, interests, or other characteristics of individuals " +
      "and combine it with identified data for the purpose of research, " +
      "analysis and reporting. For example, an online Web site for a " +
      "physical store may wish to analyze how online shoppers make " +
      "offline purchases.</span>",
    "individual-decision" :
      "Individual Decision<span>:  Information may be used to determine " +
      "the habits, interests, or other characteristics of individuals " +
      "and combine it with identified data to make a decision that " +
      "directly affects that individual.  For example, an online store " +
      "suggests items a visitor may wish to purchase based on items he " +
      "has purchased during previous visits to the Web site.</span>",
    "contact" :
      "Contacting Visitors for Marketing of Services or Products<span>: " +
      "Information may be used to contact the individual, through a " +
      "communications channel other than voice telephone, for the " +
      "promotion of a product or service. This includes notifying " +
      "visitors about updates to the Web site. This does not include a " +
      "direct reply to a question or comment or customer service for a " +
      "single transaction -- in those cases, <current/> would be used. " +
      "In addition, this does not include marketing via customized Web " +
      "content or banner advertisements embedded in sites the user is " +
      "visiting.</span>",
    "historical" :
      "Historical Preservation<span>: Information may be archived or " +
      "stored for the purpose of preserving social history as governed " +
      "by an existing law or policy. This law or policy MUST be " +
      "referenced in the <DISPUTES> element and MUST include a specific " +
      "definition of the type of qualified researcher who can access the " +
      "information, where this information will be stored and specifically " +
      "how this collection advances the preservation of history.</span>",
    "telemarketing" :
      "Contacting Visitors for Marketing of Services or Products Via " +
      "Telephone<span>: Information may be used to contact the individual " +
      "via a voice telephone call for promotion of a product or service. " +
      "This does not include a direct reply to a question or comment or " +
      "customer service for a single transaction.</span>",
    "other-purpose" : "Additional purposes."
  },

  recipients: {
    "ours" :
      "Ourselves and/or entities acting as our agents or entities for " +
      "whom we are acting as an agent<span>: An agent in this instance " +
      "is defined as a third party that processes data only on behalf " +
      "of the service provider for the completion of the stated purposes. " +
      "(e.g., the service provider and its printing bureau which prints " +
      "address labels and does nothing further with the information.)</span>",
    "delivery" :
      "Delivery services possibly following different practices<span>: " +
      "Legal entities performing delivery services that may use data for " +
      "purposes other than completion of the stated purpose. This should " +
      "also be used for delivery services whose data practices are " +
      "unknown.</span>",
    "same" :
      "Legal entities following our practices<span>: Legal entities who " +
      "use the data on their own behalf under equable practices. (e.g., " +
      "consider a service provider that grants the user access to collected " +
      "personal information, and also provides it to a partner who uses " +
      "it once but discards it. Since the recipient, who has otherwise " +
      "similar practices, cannot grant the user access to information " +
      "that it discarded, they are considered to have equable " +
      "practices.)</span>",
    "other-recipient" :
      "Legal entities following different practices<span>: Legal entities "+
      "that are constrained by and accountable to the original service " +
      "provider, but may use the data in a way not specified in the " +
      "service provider's practices (e.g., the service provider collects " +
      "data that is shared with a partner who may use it for other " +
      "purposes. However, it is in the service provider's interest to " +
      "ensure that the data is not used in a way that would be considered " +
      "abusive to the users' and its own interests.)</span>",
    "unrelated" :
      "Unrelated third parties<span>: Legal entities whose data usage " +
      "practices are not known by the original service provider.</span>",
    "public" :
      "Public fora<span>: Public fora such as bulletin boards, public " +
      "directories, or commercial CD-ROM directories.</span>"
  },

  retention: {
    "no-retention" :
      "Information is not retained for more than a brief period of time " +
      "necessary to make use of it during the course of a single online " +
      "interaction. <span>Information MUST be destroyed following this " +
      "interaction and MUST NOT be logged, archived, or otherwise stored. " +
      "This type of retention policy would apply, for example, to services " +
      "that keep no Web server logs, set cookies only for use during a " +
      "single session, or collect information to perform a search but do " +
      "not keep logs of searches performed.</span>",
    "stated-purpose" :
      "Information is retained to meet the stated purpose. <span>This " +
      "requires information to be discarded at the earliest time possible. " +
      "Sites MUST have a retention policy that establishes a destruction " +
      "time table. The retention policy MUST be included in or linked " +
      "from the site's human-readable privacy policy.</span>",
    "legal-requirement" :
      "As required by law or liability under applicable law<span>: " +
      "Information is retained to meet a stated purpose, but the " +
      "retention period is longer because of a legal requirement or " +
      "liability. For example, a law may allow consumers to dispute " +
      "transactions for a certain time period; therefore a business may " +
      "for liability reasons decide to maintain records of transactions, " +
      "or a law may affirmatively require a certain business to maintain " +
      "records for auditing or other soundness purposes. Sites MUST have " +
      "a retention policy that establishes a destruction time table. The " +
      "retention policy MUST be included in or linked from the site's " +
      "human-readable privacy policy.</span>",
    "business-practices" :
      "Determined by service provider's business practice<span>: " +
      "Information is retained under a service provider's stated " +
      "business practices. Sites MUST have a retention policy that " +
      "establishes a destruction time table. The retention policy MUST " +
      "be included in or linked from the site's human-readable privacy " +
      "policy.</span>",
    "indefinitely" :
      "Indefinitely<span>: Information is retained for an indeterminate " +
      "period of time. The absence of a retention policy would be " +
      "reflected under this option. Where the recipient is a public " +
      "fora, this is the appropriate retention policy.</span>"
  },

  get_description: function (name) {
    var desc = this.data_description[name];
    if (!desc) desc = name;
    return desc;
  },

  data_description: {
    "user.name" : "user's name",
    "user.employer" : "user's employer",
    "dynamic.clickstream" :"information typically found in Web " +
      "server access logs, e.g.  the IP address or hostname of the user's " +
      "computer, the URI of the resource requested, the time the request " +
      "was made, the HTTP method used in the request, the size of the " +
      "response, and the HTTP status code in the response.",
    "dynamic.http" : "additional information carried in HTTP requests",
    "dynamic.clientevents" :"information about how the user " +
      "interacts with a web page, e.g. mouse movements and clicks",
    "dynamic.searchtext" :"search terms entered by the user",
    "dynamic.interactionrecord": "additional information on " +
      "user interaction, e.g. account transactions, etc.",
    "dynamic.cookies" :"the use of web page cookies for various " +
      "purposes, e.g. session identifiers and status information"
  }
};



